/*
 * ConfigDialog.java - GUI to handle option configuration for QuickEdit
 *
 *      Copyright (c) - 2006 DrachenSoft, LLC
 *      This software may be freely distributed and modified under the GNU
 *      General Public License, Version 2 or later, as published by the
 *      Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK!
 *      The author and/or authors cannot be held responsible for any
 *      misuse or damage.
 *
 *      Note: Please add revisions as file is changed!
 *
 *      REV         DATE                DESCRIPTION
 *      -----       ----------          ----------------------------------------
 *      00.00       2006-07-28          Initial release
 *
 */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import javax.mail.*;
import javax.swing.*;
/**
 * GUI to handle option configuration for QuickEdit. This simply reads and writes
 * the configurations to a flat file.
 *
 * @author  Michael Latham
 */
public class ConfigDialog extends JDialog
{
	private JLabel m_hostLbl;
	private JLabel m_userLbl;
	private JLabel m_passLbl;
	private JLabel m_dfltToLbl;
	private JLabel m_dfltFromLbl;
	private JCheckBox m_authReqCB;
	private JTextField m_hostTF;
	private JTextField m_userTF;
	private JPasswordField m_passPF;
	private JTextField m_dfltToTF;
	private JTextField m_dfltFromTF;
	private JSeparator jSeparator1;
	private JButton m_applyBtn;
	private JButton m_closeBtn;
	private JPanel contentPane;
    private Frame m_parent;


    /**
     * Only constructor
     * @param w     Parent frame for dialog
     */
	public ConfigDialog(Frame w)
	{
		super(w);
        m_parent = w;
		initializeComponent();
        postInitialization();
		this.setVisible(true);
	}

	/**
     * Initialize GUI components
     */
    private void initializeComponent()
	{
		m_hostLbl = new JLabel();
		m_userLbl = new JLabel();
		m_passLbl = new JLabel();
		m_dfltToLbl = new JLabel();
		m_dfltFromLbl = new JLabel();
		m_authReqCB = new JCheckBox();
		m_hostTF = new JTextField();
		m_userTF = new JTextField();
		m_passPF = new JPasswordField();
		m_dfltToTF = new JTextField();
		m_dfltFromTF = new JTextField();
		jSeparator1 = new JSeparator();
		m_applyBtn = new JButton();
		m_closeBtn = new JButton();
		contentPane = (JPanel)this.getContentPane();

		//
		// m_hostLbl
		//
		m_hostLbl.setText("Host");
		//
		// m_userLbl
		//
		m_userLbl.setText("Username");
		//
		// m_passLbl
		//
		m_passLbl.setText("Password");
		//
		// m_dfltToLbl
		//
		m_dfltToLbl.setText("Default To");
		//
		// m_dfltFromLbl
		//
		m_dfltFromLbl.setText("Default From");
		//
		// m_authReqCB
		//
		m_authReqCB.setText("Authentication Required");
		m_authReqCB.setToolTipText("Check if email server requires authentication.");
		m_authReqCB.setSelected(false);
		m_authReqCB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e)
			{
				m_authReqCB_itemStateChanged(e);
			}

		});
		//
		// m_hostTF
		//
		m_hostTF.setToolTipText("Enter email server outgoing host address");
		//
		// m_userTF
		//
		m_userTF.setToolTipText("Enter username if authentication is required.");
        m_userTF.setEnabled(false);
		//
		// m_passPF
		//
		m_passPF.setToolTipText("Enter password if authentication is required.");
        m_passPF.setEnabled(false);
		//
		// m_dfltToTF
		//
		m_dfltToTF.setToolTipText("Default \"To\" email address");
		//
		// m_dfltFromTF
		//
		m_dfltFromTF.setToolTipText("Default \"From\" email address");
		//
		// jSeparator1
		//
		//
		// m_applyBtn
		//
		m_applyBtn.setText("Apply");
		m_applyBtn.setToolTipText("Click to save changes.");
		m_applyBtn.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e)
			{
				m_applyBtn_mouseClicked(e);
			}

		});
		//
		// m_closeBtn
		//
		m_closeBtn.setText("Close");
		m_closeBtn.setToolTipText("Click to close Config Dialog");
		m_closeBtn.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e)
			{
				m_closeBtn_mouseClicked(e);
			}

		});
		//
		// contentPane
		//
		contentPane.setLayout(null);
		addComponent(contentPane, m_hostLbl, 16,13,120,18);
		addComponent(contentPane, m_userLbl, 15,72,121,18);
		addComponent(contentPane, m_passLbl, 16,100,123,18);
		addComponent(contentPane, m_dfltToLbl, 14,147,124,18);
		addComponent(contentPane, m_dfltFromLbl, 14,176,124,18);
		addComponent(contentPane, m_authReqCB, 142,43,148,24);
		addComponent(contentPane, m_hostTF, 144,11,190,22);
		addComponent(contentPane, m_userTF, 144,69,190,22);
		addComponent(contentPane, m_passPF, 144,99,190,22);
		addComponent(contentPane, m_dfltToTF, 144,146,190,22);
		addComponent(contentPane, m_dfltFromTF, 144,175,190,22);
		addComponent(contentPane, jSeparator1, 12,131,325,14);
		addComponent(contentPane, m_applyBtn, 348,4,83,28);
		addComponent(contentPane, m_closeBtn, 348,37,83,28);
		//
		// ConfigDialog
		//
        Point p = QETools.getNewLocation( m_parent );
		this.setTitle("QuickEmail Options");
		this.setLocation(p);
		this.setSize(new Dimension(447, 243));
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setModal(true);
		this.setResizable(false);
	}

    /**
     * Add Component Without a Layout Manager (Absolute Positioning)
     * @param container Container to which to add component
     * @param c         Component to add to container
     * @param x         X coordinate position for component in container
     * @param y         Y coordinate position for component in container
     * @param width     width of component
     * @param height    height of component
     */ 
	private void addComponent(Container container,Component c,int x,int y,int width,int height)
	{
		c.setBounds(x,y,width,height);
		container.add(c);
	}
    
    /**
     * Extra initialization after components have been setup, like getting current options
     */
     private void postInitialization() 
     {
         LineNumberReader fr = getFileReader( "config.dat" );
         if (fr == null)
             fr = createDatFile();
         loadDatFileToScreen(fr);
     }

    /**
     * Open and return a LineNumberReader for the given filename
     * @param fileName  File path and name to open for reading - FILE MUST EXIST
     * @return Line number reader object
     */
    private LineNumberReader getFileReader( String fileName ) 
    {
        LineNumberReader fr = null;
        try {
            String sep = System.getProperty("file.separator");
            String file = "." + sep + "datafiles" + sep + fileName;
            fr = new LineNumberReader( new FileReader( file ) );
        }
        catch (Exception e) {
            System.err.println("Failed to open file " + fileName);
            System.err.println(e.getMessage());
            fr = null;
        }
        return fr;
    }
    
    /**
     * Create and return the config dat file
     * @return line reader attached to new config dat file, or null
     */
    private LineNumberReader createDatFile()
    {
        LineNumberReader fr = null;
        String file = "";
        try {
            String sep = System.getProperty("file.separator");
            file = "." + sep + "datafiles" + sep + "config.dat";
            FileWriter fw = new FileWriter(  file );
            fw.close();
        }
        catch (Exception e) {
            System.err.println("Failed to write file " + file);
            System.err.println(e.getMessage());
            fr = null;
        }
        
        fr = getFileReader( "config.dat" );
        
        return fr;
    }
    
    /**
     * Read contents of config file and display on the GUI
     * @param fr    Line number reader already attached to file
     */
    private void loadDatFileToScreen( LineNumberReader fr )
    {
        if (fr == null)
            return;
        
        String newLine = "";
        try {
            newLine = fr.readLine();
            while (newLine != null) {
                parseConfigLine( newLine );
                newLine = fr.readLine();
            }
            fr.close();
        }
        catch( Exception e ) {
            System.out.println( "Error reading config.dat: " + e.getMessage() );
            System.out.println( "Last new line: " + newLine );
        }

    }
    
    /**
     * Parse a line of the config file and add data to the GUI
     * @param line  data from file to parse
     */
    private void parseConfigLine( String line )
    {
        if (line.length() < 5)
            return;
        String prop = line.substring(0,4);
        String val  = "";
        if (line.length() > 5)
            val  = line.substring(5).trim();

        if (prop.compareTo("host") == 0)
            m_hostTF.setText(val);
        else if (prop.compareTo("user") == 0)
            m_userTF.setText(val);
        else if (prop.compareTo("pass") == 0) {
            val = QETools.decrypt(val);
            m_passPF.setText(val);
        }
        else if (prop.compareTo("dffr") == 0)
            m_dfltFromTF.setText(val);
        else if (prop.compareTo("dfto") == 0)
            m_dfltToTF.setText(val);
        else if (prop.compareTo("auth") == 0) {
            if (val.compareTo("yes") == 0)
                m_authReqCB.setSelected(true);
        }
    }

	/**
     * Check box state changed handler
     * @param e     Item event object
     */
    private void m_authReqCB_itemStateChanged(ItemEvent e)
	{
        if (e.getStateChange() == ItemEvent.SELECTED) {
            m_userTF.setEnabled(true);
            m_passPF.setEnabled(true);
        }
        else {
            m_userTF.setEnabled(false);
            m_passPF.setEnabled(false);
        }
	}

    /**
     * Apply button handler - saves configuration to file
     * @param me     Mouse event object
     */
	private void m_applyBtn_mouseClicked(MouseEvent me)
	{
        boolean ok = true;
        try {
            String sep = System.getProperty("file.separator");
            String eol = System.getProperty("line.separator");
            String file = "." + sep + "datafiles" + sep + "config.dat";
            FileWriter fw = new FileWriter(  file );
            
            String val = "host:" + m_hostTF.getText().trim() + eol;
            fw.write( val, 0, val.length() );

            val = "user:" + m_userTF.getText().trim() + eol;
            fw.write( val, 0, val.length() );

            String tmp = String.copyValueOf(m_passPF.getPassword());
            tmp = QETools.encrypt(tmp);
            val = "pass:" + tmp + eol;
            fw.write( val, 0, val.length() );

            val = "dffr:" + m_dfltFromTF.getText().trim() + eol;
            fw.write( val, 0, val.length() );

            val = "dfto:" + m_dfltToTF.getText().trim() + eol;
            fw.write( val, 0, val.length() );

            if (m_authReqCB.isSelected())
                val = "auth:yes" + eol;
            else
                val = "auth:no" + eol;
            fw.write( val, 0, val.length() );
            
            fw.close();
        }
        catch( Exception e ) {
            System.err.println("Failed to save config to file.");
            System.err.println(e.getMessage());
            ok = false;
        }

        if (ok)
            QETools.msg("Changes have been saved", this);
	}

    /**
     * Close button handler - closes dialog
     * @param e     Mouse event object
     */
	private void m_closeBtn_mouseClicked(MouseEvent e)
	{
        this.setVisible(false);
	}

    /* 
       NOTE: I've left the following, but commented out, incase there is a need
             to run this dialog by itself.  This "main" method can be uncommented
             and called from a setup jar, and can still be used from within the
             application 
     */
/*
	public static void main(String[] args)
	{
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		try
		{
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		}
		catch (Exception ex)
		{
			System.out.println("Failed loading L&F: ");
			System.out.println(ex);
		}
		final JFrame w = new JFrame("Owner Window");
		JButton btn = new JButton("Show Dialog");
		btn.addActionListener(new java.awt.event.ActionListener() {
							public void actionPerformed(java.awt.event.ActionEvent e)
							{
								//-- Create a ConfigDialog --
								new ConfigDialog(w);
								//-------------------
							}});
		JPanel p = new JPanel();
		p.add(btn);
		w.getContentPane().add(p);
		w.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		w.setSize(500,360);
		w.setLocation(150,36);
		w.setVisible(true);
	}
*/

}
