/*
 * QETools.java - Contains tools like outputting debug messages for other classes to use.
 *
 *      Copyright (c) - 2006 DrachenSoft, LLC
 *      This software may be freely distributed and modified under the GNU
 *      General Public License, Version 2 or later, as published by the
 *      Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK!
 *      The author and/or authors cannot be held responsible for any
 *      misuse or damage.
 *
 *      Note: Please add revisions as file is changed!
 *
 *      REV         DATE                DESCRIPTION
 *      -----       ----------          ----------------------------------------
 *      00.00       2006-07-28          Initial release
 *
 */
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Point;
import java.lang.*;
import java.util.*;
import javax.crypto.*;
import javax.crypto.spec.DESKeySpec;
/**
 * Contains tools like outputting debug messages for other classes to use.
 * Basically, if code is going to be written over and over in different classes,
 * this is the place to consolidate and write it once.
 *
 * @author  Michael Latham
 */
public class QETools
{
    /** Set to true if basic (non-db) debug messages should be written to std out */
    public static final boolean m_debug     = true;

    /** Attempted look and feel for the application */
    public static final String  m_lookAndFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
    
    /** Constructor, though most of this class is static */
    public QETools () {
    }
    
    private static final String m_key = "com.sun.crypto.provider.DESKey@fffe79fc";
    
    /** 
     * If basic debugging is turned on, output the message
     * @param msg   Debug message to output
     */
    public static void debugMsg( String msg ) {
        if (m_debug)
            System.out.println( msg );
    }
    
    /**
     * Determine a reasonable location for a new dialog given the parent it will belong to.
     * @param p Parent frame of a new dialog or frame
     * @return The point location for the new dialog or frame
     */
    public static Point getNewLocation( Frame p ) {
        Point pt = null;
        if (p == null) {
            pt = new Point(125, 100);
        }
        else {
            pt = p.getLocation();
            pt.translate(50,50);
        }
        return pt;
    }
    
    /**
     * Determine a reasonable location for a new dialog given the parent it will belong to.
     * @param p Parent dialog of a new dialog or frame
     * @return The point location for the new dialog or frame
     */
    public static Point getNewLocation( Dialog p ) {
        Point pt = null;
        if (p == null) {
            pt = new Point(125, 100);
        }
        else {
            pt = p.getLocation();
            pt.translate(50,50);
        }
        return pt;
    }

    /**
     * Do pop-up to display the message
     * @param val       Message to display
     * @param parent    Parent frame to hold dialog
     */
    public static void msg( String val, Frame parent )
    {
        MsgDialog msg = new MsgDialog(parent);
        msg.setType( MsgDialog.MESSAGE );
        msg.setData( val );
        msg.setVisible(true);
        msg.dispose();
    }

    /**
     * Do pop-up to display the message
     * Do pop-up to display the message
     * @param val       Message to display
     * @param parent    Parent dialog to hold dialog
     */
    public static void msg( String val, Dialog parent )
    {
        MsgDialog msg = new MsgDialog(parent);
        msg.setType( MsgDialog.MESSAGE );
        msg.setData( val );
        msg.setVisible(true);
        msg.dispose();
    }
    
    /**
     * Encrypt a string to save to file
     * @param in    String to encrypt
     * @return Encrypted string, or original if there was an error.
     */
    public static String encrypt( String in )
    {
        String out = in;
        try {
            byte[] desKeyData = getDesKeyData();
            DESKeySpec desKeySpec = new DESKeySpec(desKeyData);
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
            SecretKey secretKey = keyFactory.generateSecret(desKeySpec);
            Cipher c = Cipher.getInstance("DES");
            c.init(Cipher.ENCRYPT_MODE, secretKey);
            byte[] utf8Bytes = in.getBytes("UTF8");
            byte[] encBytes  = c.doFinal(utf8Bytes);
            out = new sun.misc.BASE64Encoder().encode(encBytes);
        }
        catch (Exception e) {
            System.err.println( "Error encrypting " + e );
        }
        return out;
    }

    /**
     * Decrypt a string to save to file
     * @param in    String to encrypt
     * @return Encrypted string, or original if there was an error.
     */
    public static String decrypt( String in )
    {
        String out = in;
        try {
            byte[] desKeyData = getDesKeyData();
            DESKeySpec desKeySpec = new DESKeySpec(desKeyData);
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
            SecretKey secretKey = keyFactory.generateSecret(desKeySpec);
            Cipher d = Cipher.getInstance("DES");
            d.init(Cipher.DECRYPT_MODE, secretKey);
            byte[] decBytes = new sun.misc.BASE64Decoder().decodeBuffer(in);
            byte[] utf8Bypes = d.doFinal(decBytes);
            out = new String(utf8Bypes, "UTF8");
        }
        catch (Exception e) {
            System.err.println( "Error encrypting " + e );
        }
        return out;
    }
    
    /**
     * Generate key base for encryption
     * @return byte array for use as key data
     */
    private static byte[] getDesKeyData() {
        String key = "DRACHENSOFT";
        byte[] keyArray = key.getBytes(); 
        return keyArray;
    }
}

