/*
 * QuickEmailFrame.java - Main frame for QuickEmail GUI
 *
 *      Copyright (c) - 2006 DrachenSoft, LLC
 *      This software may be freely distributed and modified under the GNU
 *      General Public License, Version 2 or later, as published by the
 *      Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK!
 *      The author and/or authors cannot be held responsible for any
 *      misuse or damage.
 *
 *      Note: Please add revisions as file is changed!
 *
 *      REV         DATE                DESCRIPTION
 *      -----       ----------          ----------------------------------------
 *      00.00       2006-07-28          Initial release
 *
 */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;
import javax.mail.*;
import javax.mail.internet.*;
//import com.sun.mail.smtp.*;
/**
 * Main frame for QuickEmail GUI
 *
 * @author  Michael Latham
 */
public class QuickEmailFrame extends JFrame
{
    /** Flag passed into Session for debug info during delivery */
	final boolean	DEBUG_FLAG	= true;

	/** Screen label */
    private JLabel m_fromLbl;
	/** Screen label */
	private JLabel m_toLbl;
	/** Screen label */
	private JLabel m_subjLbl;
	/** Email From text field */
    private JTextField m_fromTF;
	/** Email To text field */
	private JTextField m_toTF;
	/** Email Subject text field */
	private JTextField m_subjTF;
	/** Email Body text field */
	private JTextArea m_bodyTA;
	/** Screen component */
    private JScrollPane jScrollPane1;
	/** Button to send email */
    private JButton m_sendBtn;
	/** Button to close QuickEmail */
    private JButton m_closeBtn;
	/** Button to invoke the Options dialog */
    private JButton m_optionsBtn;
    /** Panel to hold GUI components */
	private JPanel contentPane;

    /** Email server host address */
    private String m_host;
    /** Email server requires authentication */
    private boolean m_authReq;
    /** Email server login user name */
    private String m_user;
    /** Email server login password */
    private JPasswordField m_password;

    /**
     * Default and only constructor
     */
	public QuickEmailFrame()
	{
		super();
		initializeComponent();
        postInitialization();
		this.setVisible(true);
	}

    /**
     * Initialize GUI components
     */
    private void initializeComponent()
	{
		m_fromLbl = new JLabel();
		m_toLbl = new JLabel();
		m_subjLbl = new JLabel();
		m_fromTF = new JTextField();
		m_toTF = new JTextField();
		m_subjTF = new JTextField();
		m_bodyTA = new JTextArea();
		jScrollPane1 = new JScrollPane();
		m_sendBtn = new JButton();
		m_closeBtn = new JButton();
		m_optionsBtn = new JButton();
		contentPane = (JPanel)this.getContentPane();
        m_password = new JPasswordField();

		//
		// m_fromLbl
		//
		m_fromLbl.setText("From");
		m_fromLbl.setToolTipText("Enter From email address");
		//
		// m_toLbl
		//
		m_toLbl.setText("To");
		m_toLbl.setToolTipText("Enter To email address");
		//
		// m_subjLbl
		//
		m_subjLbl.setText("Subject");
		m_subjLbl.setToolTipText("Enter email subject.");
		//
		// m_fromTF
		//
		m_fromTF.setToolTipText("Enter from address. See Options to set a default.");
		//
		// m_toTF
		//
		m_toTF.setToolTipText("Enter To address. See Options to set a default.");
		//
		// m_subjTF
		//
		m_subjTF.setToolTipText("Enter email subject.");
		//
		// m_bodyTA
		//
		m_bodyTA.setToolTipText("Enter email body.");
		m_bodyTA.setLineWrap(true);
		m_bodyTA.setWrapStyleWord(true);
		//
		// jScrollPane1
		//
		jScrollPane1.setViewportView(m_bodyTA);
		//
		// m_sendBtn
		//
		m_sendBtn.setText("Send");
		m_sendBtn.setToolTipText("Click to send email");
		m_sendBtn.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e)
			{
				m_sendBtn_mouseClicked(e);
			}

		});
		//
		// m_closeBtn
		//
		m_closeBtn.setText("Close");
		m_closeBtn.setToolTipText("Click to close QuickEmail.");
		m_closeBtn.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e)
			{
				m_closeBtn_mouseClicked(e);
			}

		});
		//
		// m_optionsBtn
		//
		m_optionsBtn.setText("Options...");
		m_optionsBtn.setToolTipText("Click to display Options dialog.");
		m_optionsBtn.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e)
			{
				m_optionsBtn_mouseClicked(e);
			}

		});
		//
		// contentPane
		//
		contentPane.setLayout(null);
		addComponent(contentPane, m_fromLbl, 10,10,60,20);
		addComponent(contentPane, m_toLbl, 10,35,60,20);
		addComponent(contentPane, m_subjLbl, 10,60,60,20);
		addComponent(contentPane, m_fromTF, 70,10,200,20);
		addComponent(contentPane, m_toTF, 70,35,200,20);
		addComponent(contentPane, m_subjTF, 70,60,200,20);
		addComponent(contentPane, jScrollPane1, 10,90,499,417);
		addComponent(contentPane, m_sendBtn, 219,526,83,28);
		addComponent(contentPane, m_closeBtn, 426,10,83,28);
		addComponent(contentPane, m_optionsBtn, 426,50,83,28);
		//
		// QuickEmailFrame
		//
		this.setTitle("QuickEmail");
		this.setLocation(new Point(0, 0));
		this.setSize(new Dimension(525, 600));
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setResizable(false);
	}

    /**
     * Add Component Without a Layout Manager (Absolute Positioning)
     * @param container Container to which to add component
     * @param c         Component to add to container
     * @param x         X coordinate position for component in container
     * @param y         Y coordinate position for component in container
     * @param width     width of component
     * @param height    height of component
     */ 
	private void addComponent(Container container,Component c,int x,int y,int width,int height)
	{
		c.setBounds(x,y,width,height);
		container.add(c);
	}

    /**
     * Extra initialization after components have been setup, like getting current options
     */
     private void postInitialization() 
     {
         LineNumberReader fr = getFileReader( "config.dat" );
         if (fr == null)
             fr = createDatFile();
         loadDatFileToScreen(fr);
         
         if (m_user.contains( System.getProperty("line.separator") )) {
             System.out.println("");
             System.out.println("User string contains a line feed character!");
             System.out.println("");
         }
         
     }

    /**
     * Open and return a LineNumberReader for the given filename
     * @param fileName  File path and name to open for reading - FILE MUST EXIST
     * @return Line number reader object
     */
    private LineNumberReader getFileReader( String fileName ) 
    {
        LineNumberReader fr = null;
        try {
            String sep = System.getProperty("file.separator");
            String file = "." + sep + "datafiles" + sep + fileName;
            fr = new LineNumberReader( new FileReader( file ) );
        }
        catch (Exception e) {
            System.err.println("Failed to open file " + fileName);
            System.err.println(e.getMessage());
            fr = null;
        }
        return fr;
    }
    
    /**
     * Create and return the config dat file
     * @return line reader attached to new config dat file, or null
     */
    private LineNumberReader createDatFile()
    {
        LineNumberReader fr = null;
        String file = "";
        try {
            String sep = System.getProperty("file.separator");
            file = "." + sep + "datafiles" + sep + "config.dat";
            FileWriter fw = new FileWriter(  file );
            fw.close();
        }
        catch (Exception e) {
            System.err.println("Failed to write file " + file);
            System.err.println(e.getMessage());
            fr = null;
        }
        
        fr = getFileReader( "config.dat" );
        
        return fr;
    }
    
    /**
     * Read contents of config file and display on the GUI
     * @param fr    Line number reader already attached to file
     */
    private void loadDatFileToScreen( LineNumberReader fr )
    {
        if (fr == null)
            return;
        
        String newLine = "";
        try {
            newLine = fr.readLine();
            while (newLine != null) {
                parseConfigLine( newLine );
                newLine = fr.readLine();
            }
            fr.close();
        }
        catch( Exception e ) {
            System.out.println( "Error reading config.dat: " + e );
            System.out.println( "Last new line: " + newLine );
            e.printStackTrace();
        }

    }
    
    /**
     * Parse a line of the config file and add data to the GUI
     * @param line  data from file to parse
     */
    private void parseConfigLine( String line )
    {
        if (line.length() < 5)
            return;
        String prop = line.substring(0,4);
        String val  = "";
        if (line.length() > 5)
            val  = line.substring(5).trim();

        if (prop.compareTo("host") == 0)
            m_host = val;
        else if (prop.compareTo("user") == 0)
            m_user = val;
        else if (prop.compareTo("pass") == 0) {
            val = QETools.decrypt(val);
            m_password.setText(val);
        }
        else if (prop.compareTo("dffr") == 0)
            m_fromTF.setText(val);
        else if (prop.compareTo("dfto") == 0)
            m_toTF.setText(val);
        else if (prop.compareTo("auth") == 0) {
            if (val.compareTo("yes") == 0)
                m_authReq = true;
        }
    }

    /**
     * Send button handler - does the work to send the email
     * @param me    Mouse event object
     */
	private void m_sendBtn_mouseClicked(MouseEvent me)
	{
		try {
			Properties props = System.getProperties();
			props.setProperty("mail.smtp.host", m_host);
			props.setProperty("mail.smtp.from", m_fromTF.getText());
			props.setProperty("mail.smtp.auth", "true");
			props.setProperty("mail.smtp.ehlo", "true");
			props.setProperty("mail.smtp.connectiontimeout",	"5000");
			props.setProperty("mail.smtp.timeout", "5000");
			props.setProperty("mail.smtp.allow8bitmime", "true");

            Session sess;
            if (m_authReq) {
                String pw = new String( m_password.getPassword() );
                QEAuthenticator qa = new QEAuthenticator( m_user, pw );
                sess = Session.getDefaultInstance(props, qa);
            }
            else {
                sess = Session.getDefaultInstance(props);
            }
			sess.setDebug(DEBUG_FLAG);

            MimeMessage msg         = new MimeMessage(sess);
            InternetAddress addr    = new InternetAddress( m_fromTF.getText(), false );
			msg.setFrom(addr);
            msg.addRecipients( Message.RecipientType.TO, m_toTF.getText());
			msg.setSubject(m_subjTF.getText());
			msg.setText( m_bodyTA.getText() );
			
			Transport t = sess.getTransport("smtp");
			t.send(msg);

			QETools.msg( "Mail sent without error.", this );
		}
		catch( Exception e ) {
            System.err.println("Failed to send message: " + e.getMessage() );
			QETools.msg( "" + e, this );
		}

	}

    /**
     * Close button handler - closes application
     * @param me    Mouse event object
     */
	private void m_closeBtn_mouseClicked(MouseEvent me)
	{
        System.exit(0);
	}

    /**
     * Options button handler - invokes Options dialog
     * @param me    Mouse event object
     */
	private void m_optionsBtn_mouseClicked(MouseEvent me)
	{
        ConfigDialog cd = new ConfigDialog(this);
        System.out.println("Returned from config dialog.");
        postInitialization();
	}

    /**
     * Instantiates class as a standalone application
     * @param args  Argument list (none at this time)
     */
	public static void main(String[] args)
	{
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		try
		{
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		}
		catch (Exception ex)
		{
			System.out.println("Failed loading L&F: ");
			System.out.println(ex);
		}
		new QuickEmailFrame();
	}


}
